#include "config.h"
#include "tasmData.h"
#include "dataObj.h"
#include "inputObj.h"
#include "tasmMath.h"

#include "allegro.h"
#include "gameConstants.h"

#include <vector>
using namespace std;

char *key_names[] =
{
   "(none)",         "KEY_A",          "KEY_B",          "KEY_C",
   "KEY_D",          "KEY_E",          "KEY_F",          "KEY_G",
   "KEY_H",          "KEY_I",          "KEY_J",          "KEY_K",
   "KEY_L",          "KEY_M",          "KEY_N",          "KEY_O",
   "KEY_P",          "KEY_Q",          "KEY_R",          "KEY_S",
   "KEY_T",          "KEY_U",          "KEY_V",          "KEY_W",
   "KEY_X",          "KEY_Y",          "KEY_Z",          "KEY_0",
   "KEY_1",          "KEY_2",          "KEY_3",          "KEY_4",
   "KEY_5",          "KEY_6",          "KEY_7",          "KEY_8",
   "KEY_9",          "KEY_0_PAD",      "KEY_1_PAD",      "KEY_2_PAD",
   "KEY_3_PAD",      "KEY_4_PAD",      "KEY_5_PAD",      "KEY_6_PAD",
   "KEY_7_PAD",      "KEY_8_PAD",      "KEY_9_PAD",      "KEY_F1",
   "KEY_F2",         "KEY_F3",         "KEY_F4",         "KEY_F5",
   "KEY_F6",         "KEY_F7",         "KEY_F8",         "KEY_F9",
   "KEY_F10",        "KEY_F11",        "KEY_F12",        "KEY_ESC",
   "KEY_TILDE",      "KEY_MINUS",      "KEY_EQUALS",     "KEY_BACKSPACE",
   "KEY_TAB",        "KEY_OPENBRACE",  "KEY_CLOSEBRACE", "KEY_ENTER",
   "KEY_COLON",      "KEY_QUOTE",      "KEY_BACKSLASH",  "KEY_BACKSLASH2",
   "KEY_COMMA",      "KEY_STOP",       "KEY_SLASH",      "KEY_SPACE",
   "KEY_INSERT",     "KEY_DEL",        "KEY_HOME",       "KEY_END",
   "KEY_PGUP",       "KEY_PGDN",       "KEY_LEFT",       "KEY_RIGHT",
   "KEY_UP",         "KEY_DOWN",       "KEY_SLASH_PAD",  "KEY_ASTERISK",
   "KEY_MINUS_PAD",  "KEY_PLUS_PAD",   "KEY_DEL_PAD",    "KEY_ENTER_PAD",
   "KEY_PRTSCR",     "KEY_PAUSE",      "KEY_ABNT_C1",    "KEY_YEN",
   "KEY_KANA",       "KEY_CONVERT",    "KEY_NOCONVERT",  "KEY_AT",
   "KEY_CIRCUMFLEX", "KEY_COLON2",     "KEY_KANJI",      "KEY_EQUALS_PAD",
   "KEY_BACKQUOTE",  "KEY_SEMICOLON",  "KEY_COMMAND",    "KEY_UNKNOWN1",
   "KEY_UNKNOWN2",   "KEY_UNKNOWN3",   "KEY_UNKNOWN4",   "KEY_UNKNOWN5",
   "KEY_UNKNOWN6",   "KEY_UNKNOWN7",   "KEY_UNKNOWN8",   "KEY_LSHIFT",
   "KEY_RSHIFT",     "KEY_LCONTROL",   "KEY_RCONTROL",   "KEY_ALT",
   "KEY_ALTGR",      "KEY_LWIN",       "KEY_RWIN",       "KEY_MENU",
   "KEY_SCRLOCK",    "KEY_NUMLOCK",    "KEY_CAPSLOCK",   "KEY_MAX"
};

char * campaignStrings [] =
  {
    "Demo Campaign - Since v024."
  };

int menuBgColor16 = makecol16 (192, 192, 192);
int menuFgColor16 = makecol16 (0, 0, 0);
int menuMgColor16 = makecol16 (128, 128, 128);

int menuBgColor24 = makecol24 (192, 192, 192);
int menuFgColor24 = makecol24 (0, 0, 0);
int menuMgColor24 = makecol24 (128, 128, 128);

int menuBgColor32 = makecol32 (192, 192, 192);
int menuFgColor32 = makecol32 (0, 0, 0);
int menuMgColor32 = makecol32 (128, 128, 128);

char gfxDescription [80] = "";
char mouseThresholdDesc [80] = "";
char charDesc [80] = "";
char campaignDesc [80] = "";

//general
int my_button_proc (int, DIALOG *, int);
void resetMenuColors (int);
void redrawMenu (DIALOG *);
dataObj * dOConfig;

//mainmenu
int quit(void);
int startGame(void);
int gfxConfig(void);
int keyConfig(void);
int mainMsg;
BITMAP * title = NULL;
int titlePicPos = 1;

//graphics config
int gfxToMain(void);
int pickScreenMode(void);
int pickDefaultScreenMode(void);
int tryToSwitchMode (int gfxDepth2, int gfxCard2, int gfxWidth2, int gfxHeight2);
int screenModeFilter (int, int, int, int);
int gfxWidth = 640;
int gfxHeight = 480;
int gfxDepth = 24;
int gfxCard = GFX_AUTODETECT_WINDOWED;

//key config
int changeButtonUp ();
int changeButtonDown ();
int changeButtonLeft ();
int changeButtonRight ();
int changeButtonDefense ();
int changeButtonMenu ();
int changeButtonPause ();
int changeButtonQuit ();
int changeButton(int keyIndex);
int defaultControls ();
vector <int> * keys;
int updateThresholdSlider (void *dp3, int d2);
int mouse_proc(int msg, DIALOG *d, int c);
fixed drawAngle = ftofix (0.0);
volatile int tickPassed = 0;
void incTick();
BITMAP * thresholdTemp;
int thresholdColor;
int thresholdTimer;

//start menu
int startConfig ();
char *campaign_listbox(int index, int *list_size);
int charSelect (void);
int poleon_select_proc (int, DIALOG *, int);
int froc;
int charID;
int campID;

DIALOG mainMenuDialog [] = 
  {
    /* (dialog proc) (x) (y) (w) (h) (fg) (bg) (key) (flags) (d1) (d2) (dp) (dp2) (dp3) */
    { d_clear_proc, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },
    { d_bitmap_proc, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },
    { my_button_proc, 100, 340, 200, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Start", NULL, (void *)startConfig },
    { my_button_proc, 100, 370, 200, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Graphics Settings", NULL, (void *)gfxConfig },
    { my_button_proc, 100, 400, 200, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Controls", NULL, (void *)keyConfig },
    { my_button_proc, 100, 430, 200, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Quit", NULL, (void *)quit },
    { d_yield_proc, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL},
    { NULL, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL  }
  };

DIALOG gfxConfigDialog [] = 
  {
    /* (dialog proc) (x) (y) (w) (h) (fg) (bg) (key) (flags) (d1) (d2) (dp) (dp2) (dp3) */
    { d_clear_proc, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },
    { d_text_proc, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, gfxDescription, NULL, NULL },
    { my_button_proc, 0, 20, 200, 20, 0, 0,  0, D_EXIT, 0, 0, (char *)"Change Graphics Mode", NULL, (void *)pickScreenMode },
    { my_button_proc, 0, 55, 200, 20, 0, 0,  0, D_EXIT, 0, 0, (char *)"Default", NULL, (void *)pickDefaultScreenMode },
    { my_button_proc, 0, 90, 200, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Main Menu", NULL, (void *)gfxToMain },
    { d_yield_proc, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL},
    { NULL, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL  }
  };

DIALOG keyConfigDialog [] = 
  {
    /* (dialog proc) (x) (y) (w) (h) (fg) (bg) (key) (flags) (d1) (d2) (dp) (dp2) (dp3) */
    { d_clear_proc, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },
    { my_button_proc, 100, 285, 200, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Main Menu", NULL, (void *)gfxToMain },

    { d_box_proc, 0, 0, 200, 220, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },

    { my_button_proc, 10, 20, 80, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Up", NULL, (void *)changeButtonUp },
    { d_text_proc, 115, 25, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },

    { my_button_proc, 10, 45, 80, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Down", NULL, (void *)changeButtonDown },
    { d_text_proc, 115, 50, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },

    { my_button_proc, 10, 70, 80, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Left", NULL, (void *)changeButtonLeft },
    { d_text_proc, 115, 75, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },

    { my_button_proc, 10, 95, 80, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Right", NULL, (void *)changeButtonRight },
    { d_text_proc, 115, 100, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },

    { my_button_proc, 10, 120, 80, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Defense", NULL, (void *)changeButtonDefense },
    { d_text_proc, 115, 125, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },

    { my_button_proc, 10, 145, 80, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Restart", NULL, (void *)changeButtonMenu },
    { d_text_proc, 115, 150, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },

    { my_button_proc, 10, 170, 80, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Pause", NULL, (void *)changeButtonPause },
    { d_text_proc, 115, 175, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },

    { my_button_proc, 10, 195, 80, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Exit", NULL, (void *)changeButtonQuit },
    { d_text_proc, 115, 200, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },

    { my_button_proc, 100, 260, 200, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Default Controls", NULL, (void *)defaultControls },

    { d_text_proc, 5, 5, 0, 0, 0, 0, 0, 0, 0, 0, (char *)"Keyboard", NULL, NULL },

    { d_box_proc, 200, 0, 200, 220, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },

    { d_text_proc, 205, 5, 0, 0, 0, 0, 0, 0, 0, 0, (char *)"Mouse", NULL, NULL },
    { d_text_proc, 205, 25, 0, 0, 0, 0, 0, 0, 0, 0, (char *)"Sensitivity", NULL, NULL },

    //24 = slider
    { d_slider_proc, 205, 40, 160, 20, 0, 0, 0, 0, 100, 0, NULL, (void *)updateThresholdSlider, NULL },

    { d_rtext_proc, 395, 45, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },

    { mouse_proc, 225, 60, 150, 150, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL},

    { d_textbox_proc, 0, 220, 400, 35, 0, 0, 0, 0, 0, 0, (char *)"White arrow = above threshold. Black arrow = below threshold.", NULL, NULL },

    { d_yield_proc, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL},
    { NULL, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL  }
  };

DIALOG startConfigDialog [] =
  {
    { d_clear_proc, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },
    { d_box_proc, 0, 0, 400, 200, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },
    { d_box_proc, 0, 200, 400, 200, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },
    //3
    { my_button_proc, 315, 355, 80, 40, 0, 0, 0, D_EXIT, 0, 0, (char *)"GO!!!", NULL, (void *)startGame },
    { my_button_proc, 225, 375, 80, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Main Menu", NULL, (void *)gfxToMain },
    //5
    { d_bitmap_proc, 5, 25, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },
    //6
    { d_textbox_proc, 160, 100, 230, 95, 0, 0, 0, 0, 0, 0, charDesc, NULL, NULL },
    { d_list_proc, 5, 220, 305, 150, 0, 0, 0, 0, 0, 0, (void *)campaign_listbox, NULL,  NULL},
    { d_text_proc, 5, 5, 0, 0, 0, 0, 0, 0, 0, 0, (char *)"Choose your Character", NULL, NULL },
    { d_text_proc, 5, 205, 0, 0, 0, 0, 0, 0, 0, 0, (char *)"Choose your Campaign", NULL, NULL },
    //10
    { poleon_select_proc, 160, 20, 160, 19, 0, 0, 0, 0, 0, 0, (char *)"Poleon", NULL, NULL},
    { d_yield_proc, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL},
    { NULL, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL}
  };

//class functions
config :: config ()
{
  mainMsg = mainConfigContinue;
  resetMenuColors (get_color_depth ());
}

void config :: setGfx (DATAFILE * df)
{
  title = (BITMAP *)df[mainTitle].dat;

  mainMenuDialog[titlePicPos].h = title -> h;
  mainMenuDialog[titlePicPos].w = title -> w;
  mainMenuDialog[titlePicPos].dp = title;

  //store array of pictures here
  startConfigDialog [5].dp = (BITMAP *)df[missingCharPhoto].dat;
  startConfigDialog [5].w = ((BITMAP *)df[missingCharPhoto].dat) -> w;
  startConfigDialog [5].h = ((BITMAP *)df[missingCharPhoto].dat) -> h;
}

int config :: go (dataObj * dataO)
{
  if (dOConfig == NULL && dataO != NULL)
    {
      dOConfig = dataO;
      //set the graphics variables now

      gfxWidth = dOConfig -> getVideoSettings () -> getResolutionX ();
      gfxHeight = dOConfig -> getVideoSettings () -> getResolutionY ();
      gfxDepth = dOConfig -> getVideoSettings () -> getColorDepth ();
      gfxCard = dOConfig -> getVideoSettings () -> getGfxDriver ();

      sprintf (gfxDescription, "%dx%d at %d-bit color", gfxWidth, gfxHeight, gfxDepth);
      
      resetMenuColors (gfxDepth);

      //key config variables
      keys = dOConfig -> getInputObj () -> getKeyMap ();

      keyConfigDialog[4].dp = key_names [keys -> at (keyUp)];
      keyConfigDialog[6].dp = key_names [keys -> at (keyDown)];
      keyConfigDialog[8].dp = key_names [keys -> at (keyLeft)];
      keyConfigDialog[10].dp = key_names [keys -> at (keyRight)];
      keyConfigDialog[12].dp = key_names [keys -> at (keyDefense)];
      keyConfigDialog[14].dp = key_names [keys -> at (keyMenu)];
      keyConfigDialog[16].dp = key_names [keys -> at (keyPause)];
      keyConfigDialog[18].dp = key_names [keys -> at (keyQuit)];

      sprintf (mouseThresholdDesc, "%d", dOConfig -> getInputObj () -> getControlThreshold ());

      keyConfigDialog[25].dp = mouseThresholdDesc;

      keyConfigDialog[24].d2 = dOConfig -> getInputObj () -> getControlThreshold ();

      LOCK_VARIABLE(tickPassed);
      LOCK_FUNCTION(incTick);
      install_int_ex(incTick, BPS_TO_TIMER(30));
      thresholdTemp = create_bitmap (keyConfigDialog[26].w, keyConfigDialog[26].h);

      //start game
      startConfigDialog [5].dp = (BITMAP *)(dOConfig->getDatafile ()) [poleonPhoto].dat;
      startConfigDialog [5].w = ((BITMAP *)(dOConfig->getDatafile ())[poleonPhoto].dat) -> w;
      startConfigDialog [5].h = ((BITMAP *)(dOConfig->getDatafile ())[poleonPhoto].dat) -> h;

      sprintf (charDesc, "Poleon is the overpowered 2nd-in-command of LongArms' army.");
    }

  if (dataO == NULL)
    resetMenuColors (get_color_depth ());

  mainMsg = mainConfigContinue;

  //do dialog, return message
  centre_dialog (mainMenuDialog);

  /* do the dialog */
  do_dialog(mainMenuDialog, 2);

  return mainMsg;
}

void incTick()
{
  tickPassed++;
}
END_OF_FUNCTION(incTick);

config :: ~config ()
{
  destroy_bitmap (thresholdTemp);
}


//general functions
int my_button_proc (int msg, DIALOG *d, int c)
{
   int ret = d_button_proc (msg, d, c);
   if (ret == D_CLOSE && d->dp3)
      return ((int (*)(void))d->dp3)();
   return ret;
}

void resetMenuColors (int depth)
{
  //reset colors for new color depth
  switch (depth)
    {
    case 16:
      gui_fg_color = menuFgColor16;
      gui_mg_color = menuMgColor16;
      gui_bg_color = menuBgColor16;
      break;
    case 24:
      gui_fg_color = menuFgColor24;
      gui_mg_color = menuMgColor24;
      gui_bg_color = menuBgColor24;
      break;
    case 32:
      gui_fg_color = menuFgColor32;
      gui_mg_color = menuMgColor32;
      gui_bg_color = menuBgColor32;
      break;
    }

  redrawMenu (NULL);
  return;
}

void redrawMenu (DIALOG * d)
{
  //sets colors for all dialogs.
  //centers all dialogs.
  //redraws d (unless d is NULL.)

  if (d != NULL)
    {
      dialog_message (d, MSG_DRAW, 0, NULL);
      show_mouse (screen);
    }

  centre_dialog (mainMenuDialog);
  centre_dialog (gfxConfigDialog);
  centre_dialog (keyConfigDialog);
  centre_dialog (startConfigDialog);

  set_dialog_color (mainMenuDialog, gui_fg_color, gui_bg_color);
  set_dialog_color (gfxConfigDialog, gui_fg_color, gui_bg_color);
  set_dialog_color (keyConfigDialog, gui_fg_color, gui_bg_color);
  set_dialog_color (startConfigDialog, gui_fg_color, gui_bg_color);
}

//mainMenu
int quit(void)
{
   if (alert("Really Quit?", NULL, NULL, "&Yes", "&No", 'y', 'n') == 1)
     {
       mainMsg = mainQuit;
       return D_CLOSE;
     }
   else
     {
       mainMsg = mainConfigContinue;
       return D_O_K;
     }
}

int startGame(void)
{
  mainMsg = mainBattleRestart;
  return D_CLOSE;
}

int gfxConfig(void)
{
  //run the graphics config window.
  centre_dialog (gfxConfigDialog);
  do_dialog(gfxConfigDialog, 4);
  //center all dialogs afterwards. The screen resolution may change.
  redrawMenu (NULL);

  //tell the mainMenu to redraw itself. This will turn off the mouse, so restart it.
  redrawMenu (mainMenuDialog);
  return D_O_K;
}

int keyConfig(void)
{
  //run the graphics config window.
  centre_dialog (keyConfigDialog);
  do_dialog(keyConfigDialog, 1);

  //tell the mainMenu to redraw itself. This will turn off the mouse, so restart it.
  redrawMenu (mainMenuDialog);
  return D_O_K;
}

//graphics config
int gfxToMain(void)
{
  //tell the dataObject to save the settings
  dOConfig -> saveConfig ();

  return D_CLOSE;  
}

int pickScreenMode(void)
{
  int gfxCard2 = gfxCard;
  int gfxWidth2 = gfxWidth;
  int gfxHeight2 = gfxHeight;
  int gfxDepth2 = gfxDepth;
  int gfxSetResponse = 0;

  gfxSetResponse = gfx_mode_select_filter(&gfxCard2, &gfxWidth2, &gfxHeight2, &gfxDepth2, screenModeFilter);

  //if 0, user cancelled process
  if (gfxSetResponse != 0)
    {
      tryToSwitchMode (gfxDepth2, gfxCard2, gfxWidth2, gfxHeight2);
    }
  return D_O_K;
}

int pickDefaultScreenMode(void)
{
  int gfxCard2 = GFX_AUTODETECT_WINDOWED;
  int gfxWidth2 = 640;
  int gfxHeight2 = 480;
  int gfxDepth2 = 16;

  tryToSwitchMode (gfxDepth2, gfxCard2, gfxWidth2, gfxHeight2);

  //OK, it worked. Copy the variables over.
  gfxCard = gfxCard2;
  gfxWidth = gfxWidth2;
  gfxHeight = gfxHeight2;
  gfxDepth = gfxDepth2;
  
  sprintf (gfxDescription, "%dx%d at %d-bit color", gfxWidth, gfxHeight, gfxDepth);
      
  resetMenuColors (gfxDepth);
  return D_O_K;
}

int tryToSwitchMode (int gfxDepth2, int gfxCard2, int gfxWidth2, int gfxHeight2)
{
  int gfxSetResponse = 1;

  //try to set the mode and cancel if it fails
  set_color_depth (gfxDepth2);
  gfxSetResponse = set_gfx_mode(gfxCard2, gfxWidth2, gfxHeight2, 0, 0);
  if (gfxSetResponse != 0 || gfxDepth2 == 8)
    {
      //! error!
      set_color_depth (gfxDepth);
      set_gfx_mode(gfxCard, gfxWidth, gfxHeight, 0, 0);

      resetMenuColors (gfxDepth);
      
      if (gfxDepth2 != 8)
	alert("These graphics settings don't work.", "Try a different set.", NULL, "&OK", NULL, 'o', 0);
      else
	alert("TASM does not support 8-bit color depth.", "Try a different color depth.", NULL, "&OK", NULL, 'o', 0);
      
      redrawMenu (gfxConfigDialog);
      show_mouse (screen);
    }
  else
    {
      //OK, it worked. Copy the variables over.
      gfxCard = gfxCard2;
      gfxWidth = gfxWidth2;
      gfxHeight = gfxHeight2;
      gfxDepth = gfxDepth2;
      
      sprintf (gfxDescription, "%dx%d at %d-bit color", gfxWidth, gfxHeight, gfxDepth);

      resetMenuColors (gfxDepth);

      alert("New graphics mode set!", NULL, NULL, "&OK", NULL, 'o', 0);

      //tell videosettings about it!
      dOConfig -> getVideoSettings () -> setResolution (gfxWidth, gfxHeight);
      dOConfig -> getVideoSettings () -> setColorDepth (gfxDepth);
      dOConfig -> getVideoSettings () -> setGfxDriver (gfxCard);

      redrawMenu (gfxConfigDialog);
    }

  return D_O_K;
}

int screenModeFilter (int c, int w, int h, int d)
{
  //this is used by gfx_mode_select_filter. It will return 0 (and will be included in the screen mode selection if:
  //it's not 8-bit color Depth. (no palettes!)
  //it's at least 400 pixels wide and 400 pixels tall
  if (d == 8 || w < 400 || h < 400)
    return 1;

  return 0;
}

//key config
int changeButtonUp ()
{
  return changeButton(keyUp);
}

int changeButtonDown ()
{
  return changeButton(keyDown);
}

int changeButtonLeft ()
{
  return changeButton(keyLeft);
}

int changeButtonRight ()
{
  return changeButton(keyRight);
}

int changeButtonDefense ()
{
  return changeButton(keyDefense);
}

int changeButtonMenu ()
{
  return changeButton(keyMenu);
}

int changeButtonPause ()
{
  return changeButton(keyPause);
}

int changeButtonQuit ()
{
  return changeButton(keyQuit);
}

int changeButton(int keyIndex)
{
  int readkeyReturn;

  //capture next keypress
  if (screen != NULL)
    {
      rectfill (screen, gfxWidth/2 - 60, gfxHeight/2 - 10, gfxWidth/2 + 60, gfxHeight/2 + 10, makecol (255, 255, 255));

      rect (screen, gfxWidth/2 - 60, gfxHeight/2 - 10, gfxWidth/2 + 60, gfxHeight/2 + 10, makecol (0, 0, 0));
      
      textout_centre_ex(screen, font, "Press any key", gfxWidth/2, gfxHeight/2 - 5, makecol (0, 0, 0), makecol (255, 255, 255));
    }

  readkeyReturn = readkey ();

  switch (keyIndex)
    {
    case (keyUp):
      keyConfigDialog[4].dp = key_names[readkeyReturn>>8];
      keys -> at (keyUp) = readkeyReturn>>8;
      break;
    case (keyDown):
      keyConfigDialog[6].dp = key_names[readkeyReturn>>8];
      keys -> at (keyDown) = readkeyReturn>>8;
      break;
    case (keyLeft):
      keyConfigDialog[8].dp = key_names[readkeyReturn>>8];
      keys -> at (keyLeft) = readkeyReturn>>8;
      break;
    case (keyRight):
      keyConfigDialog[10].dp = key_names[readkeyReturn>>8];
      keys -> at (keyRight) = readkeyReturn>>8;
      break;
    case (keyDefense):
      keyConfigDialog[12].dp = key_names[readkeyReturn>>8];
      keys -> at (keyDefense) = readkeyReturn>>8;
      break;
    case (keyMenu):
      keyConfigDialog[14].dp = key_names[readkeyReturn>>8];
      keys -> at (keyMenu) = readkeyReturn>>8;
      break;
    case (keyPause):
      keyConfigDialog[16].dp = key_names[readkeyReturn>>8];
      keys -> at (keyPause) = readkeyReturn>>8;
      break;
    case (keyQuit):
      keyConfigDialog[18].dp = key_names[readkeyReturn>>8];
      keys -> at (keyQuit) = readkeyReturn>>8;
      break;
    default:
      break;
    }
  dOConfig -> getInputObj () -> setKeyMap (keys);
  redrawMenu (keyConfigDialog);
  return D_O_K;
}

int defaultControls()
{
  keyConfigDialog[4].dp = key_names[keyBindUp];
  keys -> at (keyUp) = keyBindUp;

  keyConfigDialog[6].dp = key_names[keyBindDown];
  keys -> at (keyDown) = keyBindDown;
  
  keyConfigDialog[8].dp = key_names[keyBindLeft];
  keys -> at (keyLeft) = keyBindLeft;
  
  keyConfigDialog[10].dp = key_names[keyBindRight];
  keys -> at (keyRight) = keyBindRight;
  
  keyConfigDialog[12].dp = key_names[keyBindDefense];
  keys -> at (keyDefense) = keyBindDefense;
  
  keyConfigDialog[14].dp = key_names[keyBindMenu];
  keys -> at (keyMenu) = keyBindMenu;
  
  keyConfigDialog[16].dp = key_names[keyBindPause];
  keys -> at (keyPause) = keyBindPause;
  
  keyConfigDialog[18].dp = key_names[keyBindQuit];
  keys -> at (keyQuit) = keyBindQuit;
  
  dOConfig -> getInputObj () -> setKeyMap (keys);
  redrawMenu (keyConfigDialog);
  return D_O_K;
}

int updateThresholdSlider (void *dp3, int d2)
{
  if (d2 < 100)
    sprintf (mouseThresholdDesc, " %d", d2);
  else
    sprintf (mouseThresholdDesc, "%d", d2);

  keyConfigDialog[25].dp = mouseThresholdDesc;

  dialog_message (&(keyConfigDialog[25]), MSG_DRAW, 0, NULL);
  show_mouse (screen);

  dOConfig -> getInputObj () -> setControlThreshold (d2);

  return D_O_K;
}

int mouse_proc(int msg, DIALOG *d, int c)
{
  int cursorX = 0, cursorY = 0;
  fixed micX, micY, micDist;
  fixed micAngle, turnDist;
  int ct = keyConfigDialog[24].d2;
  
  switch (msg) {
  case MSG_START:
    break;
  case MSG_END:
    break;
  case MSG_IDLE:
    //this is all taken from the playerAim subobject in dumbUnit.h

    //if 1/30 second has passed

    if (tickPassed > 0)
      {
	tickPassed = 0;
	thresholdTimer--;

	get_mouse_mickeys (&cursorX, &cursorY);
    
	micX = itofix (-1 * cursorX);
	micY = itofix (cursorY);
	micDist = fixhypot (micX, micY);

	if (micDist > itofix (1))
	  {
	    if (cursorY == 0 && cursorX < 0)
	      micAngle = itofix (-64);
	    else if (cursorY == 0  && cursorX > 0)
	      micAngle = itofix (64);
	    else
	      micAngle = fixatan (fixdiv (micX, micY));

	    if (micY > itofix (0))
	      micAngle -= itofix (128);
	      	
	    micAngle = micAngle ^ 256;

	    if (micDist > itofix (ct))
	      {
		drawAngle = micAngle;
		thresholdColor = makecol (255, 255, 255);
		thresholdTimer = 5;
	      }
	    else
	      {
		turnDist = angleDist (micAngle, drawAngle);	    
		drawAngle += turnDist / 10;
		if (thresholdTimer <= 0)
		  {
		    thresholdColor = makecol (0, 0, 0);
		  }
	      }

	    //now redraw the image
	    object_message(d, MSG_DRAW, 0);
	  }
      }
    break;
  case MSG_DRAW:
    //draw here
    clear_to_color (thresholdTemp, makecol (128,128,128));

    line (thresholdTemp, d->w/2, d->h/2, (d->w/2) + fixtoi((d->w/3)*fixsin(drawAngle)), (d->h/2) - fixtoi((d->w/3)*fixcos(drawAngle)), thresholdColor);

    blit(thresholdTemp, screen, 0, 0, d->x, d->y, d->w, d->h);    
    break;
  }
  return D_O_K;
}

int startConfig ()
{
  //run the graphics config window.
  centre_dialog (startConfigDialog);
  do_dialog(startConfigDialog, 3);

  //tell the mainMenu to redraw itself. This will turn off the mouse, so restart it.
  if (mainMsg != mainBattleRestart)
    {
      redrawMenu (mainMenuDialog);
      return D_O_K;
    }
  else
    {
      //update stuff in dataObj here
      return D_CLOSE;
      //note how this is D_CLOSE, which tells the main dialog to close, as well.
    }
}

char * campaign_listbox(int index, int *list_size)
{
   if (index < 0) {
      *list_size = 1;
      return NULL;
   }
   else
     return campaignStrings[index];
}

int poleon_select_proc (int msg, DIALOG *d, int c)
{
   int ret = d_radio_proc (msg, d, c);
   DATAFILE * df = dOConfig -> getDatafile ();

   if (msg == MSG_CLICK)
     {       
       //replace pic, text
       startConfigDialog [5].dp = (BITMAP *)df[poleonPhoto].dat;
       startConfigDialog [5].w = ((BITMAP *)df[poleonPhoto].dat) -> w;
       startConfigDialog [5].h = ((BITMAP *)df[poleonPhoto].dat) -> h;

       object_message(&startConfigDialog [5], MSG_DRAW, 0);

       sprintf (charDesc, "Poleon is the overpowered 2nd-in-command of LongArms' army.");
       object_message(&startConfigDialog [6], MSG_DRAW, 0);
     }
   return ret;
}
