#include "gameConstants.h"
#include "allegro.h"
#include "inputObj.h"
#include "videoSettings.h"
#include "arena.h"
#include "hitbox.h"

#include "dataObj.h"
#include <fstream>
#include <iostream>

#include <vector>
using namespace std;

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

dataObj :: dataObj ()
{
  inpObj = new inputObj ();
  vS = new videoSettings ();
  aren = new arena ();
  firstRoundOfCampaign = 0;
  charID = 0;
}

void dataObj :: readConfig ()
{
  //reads the configuration file (tasmConfig.txt) and reads parameters.
  //if there is no file, the objects load their own defaults.
  fstream configFile;
  string strRead;
  int intRead;
  int resx = vS -> getResolutionX (), resy = vS -> getResolutionY ();
  bool resChange = false;

  bool keychange = false;
  vector <int> * keyMapChange = new vector <int> (keyNumberOfKeys, -1);

  //read config file; get size defaults
  
  configFile.open ("tasmConfig.txt", ios::in);
  
  if (configFile.is_open ())
    {
      while (!configFile.eof())
	{
	  configFile >> strRead;

	  if (strRead.compare ("resX") == 0)
	    {
	      //resolution width.
	      configFile >> intRead;
	      resx = intRead;
	      resChange = true;
	    }
	  if (strRead.compare ("resY") == 0)
	    {
	      //resolution width.
	      configFile >> intRead;
	      resy = intRead;
	      resChange = true;
	    }
	  else if (strRead.compare ("colorDepth") == 0)
	    {
	      //color depth. 15, 16, 24, and 32 are accepted.
	      configFile >> intRead;
	      
	      if (intRead == 15)
		{vS -> setColorDepth (15);}
	      else if (intRead == 32)
		{vS -> setColorDepth (32);}
	      else if (intRead == 24)
		{vS -> setColorDepth (24);}
	      else
		{vS -> setColorDepth (16);}
	    }
	  else if (strRead.compare ("gfxDriver") == 0)
	    {
	      //graphics driver, allegro specific.
	      configFile >> intRead;
	      
	      vS -> setGfxDriver (intRead);
	    }
	  else if (strRead.compare ("controlThreshold") == 0)
	    {
	      //aim control style. See dumbunit.cpp -> aimunit for more specifics
	      configFile >> intRead;
	      inpObj -> setControlThreshold (intRead);
	    }
	  //keychange. This alters one of the given keys based on Allegro's scancode format.
	  else if (strRead.compare ("keyUp") == 0)
	    {
	      configFile >> intRead;
	      keyMapChange -> at (keyUp) = intRead;
	      keychange = true;
	    }
	  else if (strRead.compare ("keyDown") == 0)
	    {
	      configFile >> intRead;
	      keyMapChange -> at (keyDown) = intRead;
	      keychange = true;
	    }
	  else if (strRead.compare ("keyLeft") == 0)
	    {
	      configFile >> intRead;
	      keyMapChange -> at (keyLeft) = intRead;
	      keychange = true;
	    }
	  else if (strRead.compare ("keyRight") == 0)
	    {
	      configFile >> intRead;
	      keyMapChange -> at (keyRight) = intRead;
	      keychange = true;
	    }
	  else if (strRead.compare ("keyDefense") == 0)
	    {
	      configFile >> intRead;
	      keyMapChange -> at (keyDefense) = intRead;
	      keychange = true;
	    }
	  else if (strRead.compare ("keyMenu") == 0)
	    {
	      configFile >> intRead;
	      keyMapChange -> at (keyMenu) = intRead;
	      keychange = true;
	    }
	  else if (strRead.compare ("keyQuit") == 0)
	    {
	      configFile >> intRead;
	      keyMapChange -> at (keyQuit) = intRead;
	      keychange = true;
	    }
	  else if (strRead.compare ("campaignID") == 0)
	    {
	      configFile >> intRead;
	      setCampaignID (intRead);
	    }
	  else if (strRead.compare ("charID") == 0)
	    {
	      configFile >> intRead;
	      setCharID (intRead);
	    }
	}
      configFile.close ();
      if (keychange)
	inpObj -> setKeyMap (keyMapChange);

      if (resChange)
	  vS -> setResolution (resx, resy);
    }
}

void dataObj :: saveConfig ()
{
  //saves the system configuration in "tasmConfig.txt", overwritting all other variables.
  fstream configFile;
  vector <int> * kM = inpObj -> getKeyMap ();

  configFile.open ("tasmConfig.txt", ios::out);
  
  if (configFile.is_open ())
    { 
      configFile << "colorDepth " << vS -> getColorDepth () << endl;
      configFile << "resX " << vS -> getResolutionX () << endl;
      configFile << "resY " << vS -> getResolutionY () << endl;
      configFile << "gfxDriver " << vS -> getGfxDriver () << endl;
      configFile << "controlThreshold " << inpObj -> getControlThreshold () << endl;

      configFile << "keyUp " << kM -> at (keyUp) << endl;
      configFile << "keyDown " << kM -> at (keyDown) << endl;
      configFile << "keyLeft " << kM -> at (keyLeft) << endl;
      configFile << "keyRight " << kM -> at (keyRight) << endl;
      configFile << "keyDefense " << kM -> at (keyDefense) << endl;
      configFile << "keyMenu " << kM -> at (keyMenu) << endl;
      configFile << "keyPause " << kM -> at (keyPause) << endl;
      configFile << "keyQuit " << kM -> at (keyQuit) << endl;
      configFile << "campaignID " << getCampaignID () << endl;
      configFile << "charID " << getCharID () << endl;
    }

  configFile.close ();
}

//   videoSettings * vS;
videoSettings * dataObj :: getVideoSettings ()
{
  return vS;
}

int dataObj :: setVideoSettings (videoSettings * v)
{
  vS = v;
  return 0;
}

//   inputObj * inpObj; 
inputObj * dataObj :: getInputObj ()
{
  return inpObj;
}

int dataObj :: setInputObj (inputObj * iO)
{
  inpObj = iO;
  return 0;
}

//   arena * aren; 
arena * dataObj :: getArena ()
{
  return aren;
}

int dataObj :: setArena (arena * t)
{
  aren = t;
  return 0;
}

//DATAFILE * picData;

DATAFILE * dataObj :: getDatafile ()
{
  return picData;
}

DATAFILE * dataObj :: setDatafile (const char * filename)
{
  picData = load_datafile (filename);
  return picData;
}

int dataObj :: getFirstRoundOfCampaign ()
{
  return firstRoundOfCampaign;
}

void dataObj :: setFirstRoundOfCampaign (int froc)
{
  firstRoundOfCampaign = froc;
}

int dataObj :: getCharID ()
{
  return charID;
}

void dataObj :: setCharID (int ci)
{
  charID = ci;
}

int dataObj :: getCampaignID ()
{
  return campaignID;
}

void dataObj :: setCampaignID (int ci)
{
  campaignID = ci;
}

dataObj :: ~dataObj ()
{
  unload_datafile (picData);
}

//////////////////////////////////////////  END OF dataObj
