#include "allegro.h"

#include "videoSettings.h"

#include <vector>
using namespace std;

videoSettings :: videoSettings ()
{
  cameraX = 160;
  cameraY = 50;

  resolutionX = 640;
  resolutionY = 480;

  backgroundR = 48;
  backgroundG = 48;
  backgroundB = 48;

  bkgHue = 0.0;
  bkgSat = 1.0;

  setBkgCols (bkgHue, bkgSat);

  arenaPosX = 0;
  arenaPosY = 0;
  arenaScale = ftofix (1.0);

  hpStartX = 0;
  hpStartY = 0;
  hpSizeX = 0; 
  hpSizeY = 0;

  colorDepth = 24;
  gfxDriver = GFX_AUTODETECT_WINDOWED;
}

void videoSettings :: setObjectPositions (int arenSiX, int arenSiY)
{
  //this sets up position and size variables for the main screen, which battlemain uses.

  //first, get the scaling down, based on the arena's dimensions.
  //they should be scaled so neither one exceeds 70% of the width or the height of the resolution.

  if (arenSiX <= 0 || arenSiY <= 0)
    {
      //handling error case
      //huh? An arena must have positive dimensions.
      arenaScale = ftofix (1.0);
      arenaPosX = 0;
      arenaPosY = resolutionY;
    }
  else if (arenSiX >= ((arenSiY * resolutionX) / resolutionY))
    {
      arenaScale = ftofix ((0.70 * resolutionX) / float (arenSiX));
      arenaPosX = (resolutionX - fixtoi(arenSiX * arenaScale)) / 2;
      arenaPosY = (resolutionY - fixtoi(arenSiY * arenaScale)) / 2;
    }
  else
    {
      arenaScale = ftofix ((0.70 * resolutionY) / float (arenSiY));
      arenaPosX = (resolutionX - fixtoi(arenSiX * arenaScale)) / 2;
      arenaPosY = (resolutionY - fixtoi(arenSiY * arenaScale)) / 2;
    }

  
  //size of hp bar = 70% of width x 10% of height. It is centered along the y axis, right above the arena.
  hpStartX = (int) (resolutionX * .15);
  hpStartY = arenaPosY - 10 - (int) (resolutionY * .05);

  hpSizeX = (int) (resolutionX * .7);
  hpSizeY = (int) (resolutionY * .05);

  return;
}

//  int cameraX, cameraY;
void videoSettings :: getCamera (int & camX, int & camY)
{
  camX = cameraX;
  camY = cameraY;
}

void videoSettings :: setCamera (int camX, int camY)
{
  cameraX = camX;
  cameraY = camY;
}

//  int resolutionX, resolutionY;
void videoSettings :: getResolution (int & resX, int & resY)
{
  resX = resolutionX;
  resY = resolutionY;
}

int videoSettings :: getResolutionX ()
{
  return resolutionX;
}

int videoSettings :: getResolutionY ()
{
  return resolutionY;
}

void videoSettings :: setResolution (int resX, int resY)
{
  resolutionX = resX;
  resolutionY = resY;
}

void videoSettings :: setBkgCols (float hue, float sat)
{
  //taking the background in hue (0-360) and saturation (0-1)
  //translates them into RGB coords for the background and icon pics.

  //v = 48/256 = 0.1875
  hsv_to_rgb (hue, sat, 0.1875, &bkgR, &bkgG, &bkgB);
  hsv_to_rgb (hue, sat, 1.0, &transR, &transG, &transB);
}

void videoSettings :: getBkg (int & r, int & g, int & b)
{
  r = bkgR;
  g = bkgG;
  b = bkgB;
}

void videoSettings :: getTransCol (int & r, int & g, int & b)
{
  r = transR;
  g = transG;
  b = transB;
}

int videoSettings :: getArenaPosX ()
{return arenaPosX;}

int videoSettings :: getArenaPosY ()
{return arenaPosY;}

int videoSettings :: getHPBarStartX ()
{return hpStartX;}

int videoSettings :: getHPBarStartY ()
{return hpStartY;}

int videoSettings :: getHPBarSizeX ()
{return hpSizeX;}

int videoSettings :: getHPBarSizeY ()
{return hpSizeY;}

fixed videoSettings :: getArenaScale ()
{return arenaScale;}

int videoSettings :: getColorDepth ()
{
  return colorDepth;
}

void videoSettings :: setColorDepth (int cd)
{
  colorDepth = cd;
}

int videoSettings :: getGfxDriver ()
{
  return gfxDriver;
}

void videoSettings :: setGfxDriver (int gd)
{
  gfxDriver = gd;
}

videoSettings ::  ~videoSettings ()
{

}
